"""Textual widgets for deepagents-cli.

Import directly from submodules, e.g.:

    ```python
    from deepagents_cli.widgets.chat_input import ChatInput
    from deepagents_cli.widgets.messages import AssistantMessage
    ```
"""
